@@ -1,11 +1,8 @@
 use crate::expression::compiler::{compile, config::CompilationConfig};
-use datafusion_expr::{expr, Expr};
-use std::ops::Deref;
-use std::sync::Arc;
+use datafusion_expr::{expr, BuiltinScalarFunction, Expr, ScalarFunctionDefinition};
 use vegafusion_common::datafusion_common::DFSchema;
 use vegafusion_core::error::Result;
 use vegafusion_core::proto::gen::expression::ArrayExpression;
-use vegafusion_datafusion_udfs::udfs::array::constructor::ARRAY_CONSTRUCTOR_UDF;
 
 pub fn compile_array(
     node: &ArrayExpression,
@@ -17,8 +14,8 @@ pub fn compile_array(
         let phys_expr = compile(el, config, Some(schema))?;
         elements.push(phys_expr);
     }
-    Ok(Expr::ScalarUDF(expr::ScalarUDF {
-        fun: Arc::new(ARRAY_CONSTRUCTOR_UDF.deref().clone()),
+    Ok(Expr::ScalarFunction(expr::ScalarFunction {
+        func_def: ScalarFunctionDefinition::BuiltIn(BuiltinScalarFunction::MakeArray),
         args: elements,
     }))
 }
