@@ -7,6 +7,7 @@ use async_trait::async_trait;
 use datafusion_expr::lit;
 
 use datafusion_common::scalar::ScalarValue;
+use datafusion_common::utils::array_into_list_array;
 use datafusion_common::DFSchema;
 use datafusion_expr::{abs, floor, when, Expr};
 use float_cmp::approx_eq;
@@ -15,7 +16,6 @@ use std::sync::Arc;
 use vegafusion_common::column::{flat_col, unescaped_col};
 use vegafusion_common::data::scalar::ScalarValueHelpers;
 use vegafusion_common::datatypes::to_numeric;
-use vegafusion_core::arrow::datatypes::{DataType, Field};
 use vegafusion_core::error::{Result, VegaFusionError};
 use vegafusion_core::proto::gen::transforms::Bin;
 use vegafusion_core::task_graph::task_value::TaskValue;
@@ -52,7 +52,9 @@ impl TransformTrait for Bin {
         let bin_index =
             floor((numeric_field.clone().sub(lit(start)).div(lit(step))).add(lit(1.0e-14)))
                 .alias(bin_index_name);
-        let sql_df = sql_df.select(vec![Expr::Wildcard, bin_index]).await?;
+        let sql_df = sql_df
+            .select(vec![Expr::Wildcard { qualifier: None }, bin_index])
+            .await?;
 
         // Add column with bin start
         let bin_start = (flat_col(bin_index_name).mul(lit(step))).add(lit(start));
@@ -121,11 +123,9 @@ impl TransformTrait for Bin {
 fn compute_output_value(bin_tx: &Bin, start: f64, stop: f64, step: f64) -> Option<TaskValue> {
     let mut fname = bin_tx.field.clone();
     fname.insert_str(0, "bin_");
-
-    let fields = ScalarValue::List(
-        Some(vec![ScalarValue::from(bin_tx.field.as_str())]),
-        Arc::new(Field::new("item", DataType::Utf8, true)),
-    );
+    let fields = ScalarValue::List(Arc::new(array_into_list_array(
+        ScalarValue::iter_to_array(vec![ScalarValue::from(bin_tx.field.as_str())]).ok()?,
+    )));
 
     if bin_tx.signal.is_some() {
         Some(TaskValue::Scalar(ScalarValue::from(vec![
