@@ -117,6 +117,8 @@ impl FunctionTransformer for ToUtcTimestampSnowflakeTransform {
                     ))),
                     SqlFunctionArg::Unnamed(SqlFunctionArgExpr::Expr(ts_arg)),
                 ],
+                filter: None,
+                null_treatment: None,
                 over: None,
                 distinct: false,
                 special: false,
@@ -155,6 +157,8 @@ impl FunctionTransformer for ToUtcTimestampBigQueryTransform {
                     quote_style: None,
                 }]),
                 args: vec![SqlFunctionArg::Unnamed(SqlFunctionArgExpr::Expr(ts_arg))],
+                filter: None,
+                null_treatment: None,
                 over: None,
                 distinct: false,
                 special: false,
@@ -172,6 +176,8 @@ impl FunctionTransformer for ToUtcTimestampBigQueryTransform {
                         SqlValue::SingleQuotedString(time_zone),
                     ))),
                 ],
+                filter: None,
+                null_treatment: None,
                 over: None,
                 distinct: false,
                 special: false,
