@@ -58,6 +58,8 @@ impl FunctionTransformer for UtcTimestampToStrBigQueryTransformer {
                     SqlValue::SingleQuotedString(time_zone),
                 ))),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
@@ -75,6 +77,8 @@ impl FunctionTransformer for UtcTimestampToStrBigQueryTransformer {
                 ))),
                 SqlFunctionArg::Unnamed(SqlFunctionArgExpr::Expr(datetime_expr)),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
@@ -114,6 +118,8 @@ impl FunctionTransformer for UtcTimestampToStrDatabricksTransformer {
                         SqlValue::SingleQuotedString(time_zone),
                     ))),
                 ],
+                filter: None,
+                null_treatment: None,
                 over: None,
                 distinct: false,
                 special: false,
@@ -132,6 +138,8 @@ impl FunctionTransformer for UtcTimestampToStrDatabricksTransformer {
                     SqlValue::SingleQuotedString("y-MM-dd HH:mm:ss.SSS".to_string()),
                 ))),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
@@ -154,6 +162,8 @@ impl FunctionTransformer for UtcTimestampToStrDatabricksTransformer {
                     SqlValue::SingleQuotedString("T".to_string()),
                 ))),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
@@ -204,6 +214,8 @@ impl FunctionTransformer for UtcTimestampToStrDuckDBTransformer {
                     SqlValue::SingleQuotedString("%Y-%m-%dT%H:%M:%S.%g".to_string()),
                 ))),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
@@ -254,6 +266,8 @@ impl FunctionTransformer for UtcTimestampToStrPostgresTransformer {
                     SqlValue::SingleQuotedString("YYYY-MM-DD\"T\"HH24:MI:SS.MS".to_string()),
                 ))),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
@@ -298,6 +312,8 @@ impl FunctionTransformer for UtcTimestampToStrSnowflakeTransformer {
                     ))),
                     SqlFunctionArg::Unnamed(SqlFunctionArgExpr::Expr(ts_expr)),
                 ],
+                filter: None,
+                null_treatment: None,
                 over: None,
                 distinct: false,
                 special: false,
@@ -316,6 +332,8 @@ impl FunctionTransformer for UtcTimestampToStrSnowflakeTransformer {
                     SqlValue::SingleQuotedString("YYYY-MM-DD\"T\"HH24:MI:SS.FF3".to_string()),
                 ))),
             ],
+            filter: None,
+            null_treatment: None,
             over: None,
             distinct: false,
             special: false,
