@@ -485,12 +485,12 @@ mod test_non_finite_numbers {
 #[cfg(test)]
 mod test_scalar_math_functions {
     use crate::*;
-    use datafusion_expr::{expr, BuiltinScalarFunction, Expr};
+    use datafusion_expr::{expr, BuiltinScalarFunction, Expr, ScalarFunctionDefinition};
     use vegafusion_common::column::flat_col;
 
     fn make_scalar_fn1(fun: BuiltinScalarFunction, arg: &str, alias: &str) -> Expr {
         Expr::ScalarFunction(expr::ScalarFunction {
-            fun,
+            func_def: ScalarFunctionDefinition::BuiltIn(fun),
             args: vec![flat_col(arg)],
         })
         .alias(alias)
@@ -498,7 +498,7 @@ mod test_scalar_math_functions {
 
     fn make_scalar_fn2(fun: BuiltinScalarFunction, arg1: &str, arg2: &str, alias: &str) -> Expr {
         Expr::ScalarFunction(expr::ScalarFunction {
-            fun,
+            func_def: ScalarFunctionDefinition::BuiltIn(fun),
             args: vec![flat_col(arg1), flat_col(arg2)],
         })
         .alias(alias)
@@ -577,7 +577,7 @@ mod test_is_finite {
     use arrow::array::{Float64Array, Int32Array};
     use arrow::datatypes::{DataType, Field, Schema, SchemaRef};
     use arrow::record_batch::RecordBatch;
-    use datafusion_expr::{expr, Expr};
+    use datafusion_expr::{expr, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
 
@@ -613,13 +613,13 @@ mod test_is_finite {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(ISFINITE_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(ISFINITE_UDF.clone())),
                     args: vec![flat_col("a")],
                 })
                 .alias("f1"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(ISFINITE_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(ISFINITE_UDF.clone())),
                     args: vec![flat_col("b")],
                 })
                 .alias("f2"),
@@ -650,7 +650,7 @@ mod test_is_finite {
 #[cfg(test)]
 mod test_str_to_utc_timestamp {
     use crate::*;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::str_to_utc_timestamp::STR_TO_UTC_TIMESTAMP_UDF;
@@ -674,8 +674,10 @@ mod test_str_to_utc_timestamp {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(STR_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        STR_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("b"), lit("America/New_York")],
                 })
                 .alias("b_utc"),
@@ -712,7 +714,7 @@ mod test_str_to_utc_timestamp {
 #[cfg(test)]
 mod test_date_part_tz {
     use crate::*;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::date_part_tz::DATE_PART_TZ_UDF;
@@ -737,8 +739,10 @@ mod test_date_part_tz {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(STR_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        STR_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("b"), lit("America/New_York")],
                 })
                 .alias("b_utc"),
@@ -750,18 +754,18 @@ mod test_date_part_tz {
                 flat_col("a"),
                 flat_col("b"),
                 flat_col("b_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_PART_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_PART_TZ_UDF.clone())),
                     args: vec![lit("hour"), flat_col("b_utc"), lit("UTC")],
                 })
                 .alias("hours_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_PART_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_PART_TZ_UDF.clone())),
                     args: vec![lit("hour"), flat_col("b_utc"), lit("America/Los_Angeles")],
                 })
                 .alias("hours_la"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_PART_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_PART_TZ_UDF.clone())),
                     args: vec![lit("hour"), flat_col("b_utc"), lit("America/New_York")],
                 })
                 .alias("hours_nyc"),
@@ -801,7 +805,7 @@ mod test_date_part_tz {
 #[cfg(test)]
 mod test_date_trunc_tz {
     use crate::*;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::date_trunc_tz::DATE_TRUNC_TZ_UDF;
@@ -826,8 +830,10 @@ mod test_date_trunc_tz {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(STR_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        STR_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("b"), lit("America/New_York")],
                 })
                 .alias("b_utc"),
@@ -839,18 +845,18 @@ mod test_date_trunc_tz {
                 flat_col("a"),
                 flat_col("b"),
                 flat_col("b_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_TRUNC_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_TRUNC_TZ_UDF.clone())),
                     args: vec![lit("day"), flat_col("b_utc"), lit("UTC")],
                 })
                 .alias("day_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_TRUNC_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_TRUNC_TZ_UDF.clone())),
                     args: vec![lit("day"), flat_col("b_utc"), lit("America/Los_Angeles")],
                 })
                 .alias("day_la"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_TRUNC_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_TRUNC_TZ_UDF.clone())),
                     args: vec![lit("day"), flat_col("b_utc"), lit("America/New_York")],
                 })
                 .alias("day_nyc"),
@@ -890,7 +896,7 @@ mod test_date_trunc_tz {
 #[cfg(test)]
 mod test_make_timestamp_tz {
     use crate::*;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::make_utc_timestamp::MAKE_UTC_TIMESTAMP;
@@ -912,8 +918,8 @@ mod test_make_timestamp_tz {
         let df_result = df
             .select(vec![
                 flat_col("a"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(MAKE_UTC_TIMESTAMP.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(MAKE_UTC_TIMESTAMP.clone())),
                     args: vec![
                         flat_col("Y"),
                         flat_col("M"),
@@ -926,8 +932,8 @@ mod test_make_timestamp_tz {
                     ],
                 })
                 .alias("ts_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(MAKE_UTC_TIMESTAMP.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(MAKE_UTC_TIMESTAMP.clone())),
                     args: vec![
                         flat_col("Y"),
                         flat_col("M"),
@@ -940,8 +946,8 @@ mod test_make_timestamp_tz {
                     ],
                 })
                 .alias("ts_nyc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(MAKE_UTC_TIMESTAMP.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(MAKE_UTC_TIMESTAMP.clone())),
                     args: vec![
                         flat_col("Y"),
                         flat_col("M"),
@@ -987,7 +993,7 @@ mod test_make_timestamp_tz {
 #[cfg(test)]
 mod test_epoch_to_utc_timestamp {
     use crate::*;
-    use datafusion_expr::{expr, Expr};
+    use datafusion_expr::{expr, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::epoch_to_utc_timestamp::EPOCH_MS_TO_UTC_TIMESTAMP_UDF;
@@ -1009,8 +1015,10 @@ mod test_epoch_to_utc_timestamp {
             .select(vec![
                 flat_col("a"),
                 flat_col("t"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(EPOCH_MS_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        EPOCH_MS_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("t")],
                 })
                 .alias("t_utc"),
@@ -1047,7 +1055,7 @@ mod test_epoch_to_utc_timestamp {
 #[cfg(test)]
 mod test_utc_timestamp_to_epoch_ms {
     use crate::*;
-    use datafusion_expr::{expr, Expr};
+    use datafusion_expr::{expr, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::epoch_to_utc_timestamp::EPOCH_MS_TO_UTC_TIMESTAMP_UDF;
@@ -1071,8 +1079,10 @@ mod test_utc_timestamp_to_epoch_ms {
             .select(vec![
                 flat_col("a"),
                 flat_col("t"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(EPOCH_MS_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        EPOCH_MS_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("t")],
                 })
                 .alias("t_utc"),
@@ -1084,8 +1094,10 @@ mod test_utc_timestamp_to_epoch_ms {
                 flat_col("a"),
                 flat_col("t"),
                 flat_col("t_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(UTC_TIMESTAMP_TO_EPOCH_MS.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        UTC_TIMESTAMP_TO_EPOCH_MS.clone(),
+                    )),
                     args: vec![flat_col("t_utc")],
                 })
                 .alias("epoch_millis"),
@@ -1125,7 +1137,7 @@ mod test_utc_timestamp_to_epoch_ms {
 #[cfg(test)]
 mod test_date_add_tz {
     use crate::*;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::str_to_utc_timestamp::STR_TO_UTC_TIMESTAMP_UDF;
@@ -1149,8 +1161,10 @@ mod test_date_add_tz {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(STR_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        STR_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("b"), lit("UTC")],
                 })
                 .alias("b_utc"),
@@ -1162,13 +1176,13 @@ mod test_date_add_tz {
                 flat_col("a"),
                 flat_col("b"),
                 flat_col("b_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_ADD_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_ADD_TZ_UDF.clone())),
                     args: vec![lit("month"), lit(1), flat_col("b_utc"), lit("UTC")],
                 })
                 .alias("month_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_ADD_TZ_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(DATE_ADD_TZ_UDF.clone())),
                     args: vec![
                         lit("month"),
                         lit(1),
@@ -1213,7 +1227,7 @@ mod test_date_add_tz {
 #[cfg(test)]
 mod test_utc_timestamp_to_str {
     use crate::*;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::str_to_utc_timestamp::STR_TO_UTC_TIMESTAMP_UDF;
@@ -1238,8 +1252,10 @@ mod test_utc_timestamp_to_str {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(STR_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        STR_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("b"), lit("UTC")],
                 })
                 .alias("b_utc"),
@@ -1251,13 +1267,17 @@ mod test_utc_timestamp_to_str {
                 flat_col("a"),
                 flat_col("b"),
                 flat_col("b_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(UTC_TIMESTAMP_TO_STR_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        UTC_TIMESTAMP_TO_STR_UDF.clone(),
+                    )),
                     args: vec![flat_col("b_utc"), lit("UTC")],
                 })
                 .alias("str_utc"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(UTC_TIMESTAMP_TO_STR_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        UTC_TIMESTAMP_TO_STR_UDF.clone(),
+                    )),
                     args: vec![flat_col("b_utc"), lit("America/New_York")],
                 })
                 .alias("str_nyc"),
@@ -1300,7 +1320,7 @@ mod test_date_to_utc_timestamp {
     use arrow::array::{ArrayRef, Date32Array, Int32Array};
     use arrow::datatypes::{DataType, Field, Schema, SchemaRef};
     use arrow::record_batch::RecordBatch;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::date_to_utc_timestamp::DATE_TO_UTC_TIMESTAMP_UDF;
@@ -1332,8 +1352,10 @@ mod test_date_to_utc_timestamp {
             .select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(DATE_TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(
+                        DATE_TO_UTC_TIMESTAMP_UDF.clone(),
+                    )),
                     args: vec![flat_col("b"), lit("America/New_York")],
                 })
                 .alias("b_utc"),
@@ -1373,7 +1395,7 @@ mod test_timestamp_to_utc_timestamp {
     use arrow::array::{ArrayRef, Int32Array, TimestampMillisecondArray};
     use arrow::datatypes::{DataType, Field, Schema, SchemaRef, TimeUnit};
     use arrow::record_batch::RecordBatch;
-    use datafusion_expr::{expr, lit, Expr};
+    use datafusion_expr::{expr, lit, Expr, ScalarFunctionDefinition};
     use std::sync::Arc;
     use vegafusion_common::column::flat_col;
     use vegafusion_datafusion_udfs::udfs::datetime::to_utc_timestamp::TO_UTC_TIMESTAMP_UDF;
@@ -1404,8 +1426,8 @@ mod test_timestamp_to_utc_timestamp {
             df.select(vec![
                 flat_col("a"),
                 flat_col("b"),
-                Expr::ScalarUDF(expr::ScalarUDF {
-                    fun: Arc::new(TO_UTC_TIMESTAMP_UDF.clone()),
+                Expr::ScalarFunction(expr::ScalarFunction {
+                    func_def: ScalarFunctionDefinition::UDF(Arc::new(TO_UTC_TIMESTAMP_UDF.clone())),
                     args: vec![flat_col("b"), lit("America/New_York")],
                 })
                 .alias("b_utc"),
@@ -1445,7 +1467,7 @@ mod test_timestamp_to_utc_timestamp {
 #[cfg(test)]
 mod test_string_ops {
     use crate::*;
-    use datafusion_expr::{expr, lit, BuiltinScalarFunction, Expr};
+    use datafusion_expr::{expr, lit, BuiltinScalarFunction, Expr, ScalarFunctionDefinition};
     use vegafusion_common::column::flat_col;
 
     #[apply(dialect_names)]
@@ -1468,22 +1490,22 @@ mod test_string_ops {
                 flat_col("b"),
                 flat_col("c"),
                 Expr::ScalarFunction(expr::ScalarFunction {
-                    fun: BuiltinScalarFunction::Substr,
+                    func_def: ScalarFunctionDefinition::BuiltIn(BuiltinScalarFunction::Substr),
                     args: vec![flat_col("b"), lit(2), lit(2)],
                 })
                 .alias("b_substr"),
                 Expr::ScalarFunction(expr::ScalarFunction {
-                    fun: BuiltinScalarFunction::Concat,
+                    func_def: ScalarFunctionDefinition::BuiltIn(BuiltinScalarFunction::Concat),
                     args: vec![flat_col("b"), lit(" "), flat_col("c")],
                 })
                 .alias("bc_concat"),
                 Expr::ScalarFunction(expr::ScalarFunction {
-                    fun: BuiltinScalarFunction::Upper,
+                    func_def: ScalarFunctionDefinition::BuiltIn(BuiltinScalarFunction::Upper),
                     args: vec![flat_col("b")],
                 })
                 .alias("b_upper"),
                 Expr::ScalarFunction(expr::ScalarFunction {
-                    fun: BuiltinScalarFunction::Lower,
+                    func_def: ScalarFunctionDefinition::BuiltIn(BuiltinScalarFunction::Lower),
                     args: vec![flat_col("b")],
                 })
                 .alias("b_lower"),
